/*********************************************************************/
/* Copyright (C) 2008  NEC Research Laboratories

Authors:    Ettore Ferranti
            Ettore.Ferranti@nw.neclab.eu
	    NEC Research Laboratories
	    Heidelberg, Germany

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

#ifndef ASNETWORK_H_
#define ASNETWORK_H_

#include <vector>
#include <map>
#include <cstdio>
#include "network.h"
#include "types.h"

/// utility class, containing a pointer to a peer and the latency between a source and that peer
class PEER_LATENCY
{
public:
    PPeer peer;
    Latency latency;
    PEER_LATENCY(PPeer peer,Latency latency)
    {
        this->peer=peer;
        this->latency=latency;
    }
    bool operator<(const PEER_LATENCY &other) const
    {
        return this->latency < other.latency;
    }
};

/// utility function, used to order a list
template <typename T>
struct pointer_less_than
{
    const bool operator()(const T *a, const T * b) const
    {
        return *a < *b;
    }
};

class ASNetwork: public Network
{
	///each router id is assigned to as id
	std::map< RouterID, AS_ID > routerMap;
	///each as id has a map with <other as id, number of hops to reach it>
	std::map< AS_ID, PAS > asMap;
	///next as id
	AS_ID nextId;


public:
        ASNetwork();
        /// Compute the path latency *within the  network only* of the given path.
        /// This is defined as the latency between the two ASes to which the routers are attached.
	Latency network_latency(Path path);
	/// Assign the specified router to the specified AS
	void assignRouter(PRouter router, PAS as);
	/// returns the number of AS hops between two routers.
	Latency getHops(PRouter router1,PRouter router2);
	/// given a peer A and a list of other peers, it orders the list according to
        /// the latency between A and each one of the other peers, minimum latency first.
	std::vector<PEER_LATENCY*> orderList(PPeer source, std::vector<PPeer> list);
	/// create an AS and add it to the current network.
	PAS add_as();
	/// returns the pointer to the AS with the specified id.
	PAS as(AS_ID as);
	/// create a (bidirectional) link between the two specified ASes with the specified latency.
	void linkAs(AS_ID id1,AS_ID id2,Latency latency);
};

typedef ASNetwork* PASNetwork;
static ASNetwork* currentNetwork = NULL;

#endif /* ASNETWORK_H_ */
