#TESTH=$SRCDIRABS/test.h
SIMULATOR='$SRCDIRABS/sim'


function backend_init {
  recompile $1
}

function video_init {
  if [[ $PSNR_VIDEO ]]; then
    eval PSNR_CODER2=\"$PSNR_CODER\"
    echo "running: $PSNR_TOOLS/QoE.pl init $PSNR_VIDEO \"$PSNR_CODER2\""
    $PSNR_TOOLS/QoE.pl init $PSNR_VIDEO "$PSNR_CODER2" >${PREFIX}encode.out
    cp ${PREFIX}encode.out ${PSNR_TMPPREFIX}encode.out
    $PSNR_TOOLS/QoE.pl chunk $VIDEO_CHUNKER
  fi
}


function configure_init {
  echo >/dev/null
}




function configure_scheduler {
  PEER_TYPE="CONFIG_PEER"
  
  unset SCHEDULER_PARAMETERS

  if [ $CSCHED = "LUc" ]; then
    CS="BestChunk,Recentness"
  elif [ $CSCHED = "PLUc" ]; then
    CS="BestChunk,RecentnessPriority"
  elif [ $CSCHED = "RUc" ]; then
    CS="BestChunk,UnitWtChunk"
  elif [ $CSCHED = "PRUc" ]; then
    CS="BestChunk,Priority"
  elif [ $CSCHED = "DLc" ]; then
    CS="BestChunk,DLc"
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS DLIncrement=2"
  elif [[ $CSCHED =~ ^DLc ]] ; then
    CS="BestChunk,DLc"
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS DLIncrement=${CSCHED/DLc/}"
  else
    echo "ERROR: unsupported chunk scheduler: $CSCHED"
    exit
   fi

  if [ $PSCHED = "BAp_w" ]; then
    PS="WeightedPeer,UploadBW" 
  elif [ $PSCHED = "BAp" ]; then
    PS="BestPeer,UploadBW" 
  elif [ $PSCHED = "ELp" ]; then
    PS="BestPeer,EarliestLatest"
  elif [ $PSCHED = "TITTATp" ]; then
    PS="BestPeer,DownUpRatio"
  elif [ $PSCHED = "BAELp" ]; then
    PS="BestPeer,BAELp" 
  #elif [ $PSCHED = "U_LNNC_p" ];
  #elif [ $PSCHED = "MDp" ];
  #elif [ $PSCHED = "U_LC_p" ];
  #elif [ $PSCHED = "U_DOWNUP_p" ];
  #elif [ $PSCHED = "U_STRIPE_p" ];
  elif [ $PSCHED = "RUp" ]; then
    PS="WeightedPeer,NeedsSelected,UnitWtPeer" 
  else
    echo "ERROR: unsupported peer scheduler: $PSCHED"
    exit
  fi

  if [ $SCHED_ORDER = "C_P" ]; then
    # the filter should logically be UsefulToSomeone, but ChunkFirst selection loop anyway, so we can also use AllChunks. This will speed up things considerably!
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS"\
" ConfigPeerConfig=ChunkFirst,$CS,AllChunks,$PS,NeedsSelected"\
" ConfigPeerSourceConfig=ChunkFirst,BestChunk,Recentness,AllChunks,$PS,NeedsSelected"\
" PlayoutDelay=$PLAYOUT_DEADLINE"\
" PeerType=$PEER_TYPE"

  elif [ $SCHED_ORDER = "P_C" ]; then
    SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS"\
" ConfigPeerConfig=PeerFirst,$PS,NeedsAny,$CS,UsefulToSelected"\
" ConfigPeerSourceConfig=PeerFirst,$PS,NeedsAny,BestChunk,Recentness,UsefulToSelected"\
" PlayoutDelay=$PLAYOUT_DEADLINE"\
" PeerType=$PEER_TYPE"
  fi 

  SCHEDULER_PARAMETERS="$SCHEDULER_PARAMETERS"\
" PriorityClassCount=$PCCOUNT"\
" DLIncrDiff=$DLINCRDIFF"\

}

# configure_network $SRCDIRABS
function configure_network {
  NETWORK_PARAMETERS=""\
" NumPeers=$NUM_PEERS"\
" NumChunks=$NUM_CHUNKS"\
" StartStats=`echo a=$NUM_CHUNKS*$FIRST_CHUNK';a/1'|bc`"\
" StopStats=`echo a=$NUM_CHUNKS*$LAST_CHUNK';a/1-1'|bc`"\
" VideoRate=$VIDEORATE"\
" Degree=$DEGREE"\
" HostType=$HTYPE"\
" PeerBandwidths={{$PEERSCLASS1,$MEANCLASS1,$MEANDLCLASS1},{$PEERSCLASS2,$MEANCLASS2,$MEANDLCLASS2},{$PEERSCLASS3,$MEANCLASS3,$MEANDLCLASS3},{$PEERSCLASS4,$MEANCLASS4,$MEANDLCLASS4}}"\
" SourceUploadBW=$SOURCEUPLOADBW"\
" BandwidthFractVariation=$VARIANCERATE"\
" ProbChunkError=$PROB_CHUNK_ERROR"\
" ChunkTimeoutInterval=$CHUNK_TIMEOUT"\
" FECn=$FECN"\
" FECk=$FECK"\
""
}

function configure_other {
  if [[ $PERCENTILE ]]; then
    OTHER_PARAMETERS=""\
" DelayCdfFile=$DELAY_CDF_FILE"
  fi

  if [[ $PSNR_VIDEO ]]; then
    OTHER_PARAMETERS+=""\
" VideoStreamType=INPUT"
  fi
}

function recompile {
  cd $1
  make -j 4
  cd -
}

# params: 1:SRCDIRABS 2:seed, 3:resultsdir
function run {
  CMD="$1/sim $NETWORK_PARAMETERS $SCHEDULER_PARAMETERS $OTHER_PARAMETERS OverlaySeed=$2"
  echo running: $CMD

  cd $3

  #we need config.txt here
  cp $1/config.script.txt config.txt || exit 1
  #we need chunks.txt here
  if [[ $PSNR_VIDEO ]]; then
    cp ${PSNR_TMPPREFIX}chunks.txt chunks.txt
  fi
  # we also need a folder called "output"
  mkdir -p output
  # create repeat script
  echo $CMD >rerun.sh
  chmod a+x rerun.sh

  $1/proc-time 2>&1 $CMD | tee run.out

  # calculate PSNR values if reqired
  if [[ $PSNR_EVERY ]]; then
    $PSNR_TOOLS/QoE.pl calc output/ $VIDEO_CHUNKER $PSNR_EVERY >psnr.out
  fi
  
  cd -
}

# params: 1:SRCDIRABS 2:seed, 3:resultsdir
function evaluate {
  eval LINEHEAD2=\"$LINEHEAD\"
  RESULTSLINE=$LINEHEAD2

#  for PCLASS in 0 $PCLASSs; do
    RESULTSLINE2=$RESULTSLINE
    RESULTSLINE2+=`grep "TESTresults[[:space:]]$PCLASS[[:space:]]$PEERCLASS" $RUN_UID/run.out`
    if [[ $? == 1 ]]; then return; fi	# verify the success of the grep

    if [[ $PCLASS == "all" ]]; then
      if [[ $PSNR_EVERY ]]; then
        #RESULTSLINE2+=" "`grep "PSNRresults[[:space:]]encoded" psnr.out`
        RESULTSLINE2+=" "`grep "PSNRresults[[:space:]]$PEERCLASS" $RUN_UID/psnr.out`
        RESULTSLINE2+=" "`grep "PSNRencoded" ${PSNR_TMPPREFIX}encode.out`
      else
        RESULTSLINE2+=" PSNRresults NaN NaN NaN"
        RESULTSLINE2+=" PSNRencoded NaN NaN"
      fi
    fi

    if [[ $PEERCLASS == "all" && $PCLASS == "all" ]]; then
      RESULTSLINE2+=" "`grep user $RUN_UID/run.out`	# these are not obligatory, sometimes not even displayed if execution was fast
      RESULTSLINE2+=" "`grep MAXSIZE $RUN_UID/run.out`
      RESULTSLINE2+=" "`grep AVGSIZE $RUN_UID/run.out`
    fi

    echo -e $RESULTSLINE2 >>$TESTOUT
#  done
#  exit

  if [[ $OUTPREFIX ]]; then
    eval OUTPREFIX2=\"$OUTPREFIX\"
    cp $3/$DELAY_CDF_FILE.out $OUTPREFIX2-$DELAY_CDF_FILE.out
  fi
}


