/*********************************************************************/
/* Copyright (C) 2009 University of Trento

Authors:    Jovan Stevovic,Csaba Kiraly
			Telecomunication Networks Group
			University of Trento
			Trento, Italy
			kiraly@disi.unitn.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
    Defines the ChunkInstance class. This represents a copy of a chunk
	at a given peer.
 */

#include "chunkinstance.h"
#include "chunk.h"


ChunkInstance::ChunkInstance(ChunkID id, Time time, ChunkSize size, ChunkType type, bool par, double prio) {
    c_chunk = new Chunk(id, time, size, type, par, prio);
    c_dl = 0;
}

ChunkInstance::ChunkInstance(PChunk chunk) {
    c_chunk = chunk->chunk_pointer();
    c_dl = chunk->dl();
}

ChunkDeadline ChunkInstance::dl() {
    return c_dl;
}

void ChunkInstance::set_dl(ChunkDeadline dl) {
    c_dl = dl;
}

Time ChunkInstance::initiation_time() {
    return c_chunk->initiation_time();
}

ChunkID ChunkInstance::id() {
    return c_chunk->id();
}

void ChunkInstance::set_chunk(PChunkPointer chunk) {
    c_chunk = chunk;
}

PChunkPointer ChunkInstance::chunk_pointer() {
    return c_chunk;
}

bool ChunkInstance::parity() { 
	return c_chunk->parity();
}

double ChunkInstance::priority() { 
	return c_chunk->priority();
}
