/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file globals.cpp
	Just defines the two system-wide global variables.
 */

#include <cassert>
#include "globals.h"
#include "propertymap.h"
#include "stats.h"
#include "aopeer.h"
#include "videostream.h"

PropertyMap config;

double gProbChunkError = 0;

double  gTimeJitter = 0.000001;

// Chunk characteristics
double gChunkSize = 0.1;

// Source characteristics
double gVideoRate = 1;

// Playout Delay for sliding window
Time gPlayoutDelay = 5;

// The maximum degree of TREE overlays (TODO: can we lose this?!)
size_t gMaxTreeDegree = 5;


// The timeout on chunk delivery failure
double gChunkTimeoutInterval = 0;

// Whether to handle all contemporary deliveries before selecting what to send
bool gContemporaryDeliveries = false;

// Deadline increment value for DLc scheduler
ChunkDeadline gDLIncrement = 2;

ChunkDeadline gDLIncrDiff = 1;

// Number of priority classes
int gPriorityClassCount = 1;

Time gCurrentTime = 0;
long int gSeed = 0;

void initialize_globals() {
	gProbChunkError = config.getDouble("ProbChunkError");

	gTimeJitter = config.getDouble("TimeJitter");

	// Chunk characteristics
	gChunkSize = config.getDouble("ChunkSize");

	// Playout Delay for sliding window
	gPlayoutDelay = config.getDouble("PlayoutDelay");

	// The maximum degree of TREE overlays (TODO: can we lose this?!)
	gMaxTreeDegree = config.getSizeT("MaxTreeDegree");

	// The timeout on chunk delivery failure
	gChunkTimeoutInterval = config.getDouble("ChunkTimeoutInterval");

	// Whether to handle all contemporary deliveries before selecting what to send
	if (config.has_key("ContemporaryDeliveries")) gContemporaryDeliveries = config.getBool("ContemporaryDeliveries");

	// Deadline increment
	if (config.has_key("DLIncrement")) gDLIncrement = config.getDouble("DLIncrement");

	// Deadline increment
	if (config.has_key("DLIncrDiff")) gDLIncrDiff = config.getDouble("DLIncrDiff");

	// Priority classes
	if (config.has_key("PriorityClassCount")) gPriorityClassCount = config.getInt("PriorityClassCount");

	PeerStats::set_num_delay_counters(config.getInt("ChunkDelayCounters"));

	gSeed = config.getLong("OverlaySeed"); // The random number

	AOPeer::init_statics();  // Initialise static fields of AOPeer

	VideoStream::set_fec_n(config.getInt("FECn"));  // block size with parity

	VideoStream::set_fec_k(config.getInt("FECk"));  // block size without parity
}


