/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
    Implements the GNPOverlay, which is subclass of MBWOverlay in which
	neighbours are chosen randomly and the degree of each peer is determined
	by a poisson process.
 */

#include <cmath>
#include "gnpoverlay.h"
#include "random.h"
#include "misc.h"
#include "globals.h"

GNPOverlay::GNPOverlay(size_t num_peers, size_t degree) {
	if(degree >= num_peers) {
		fatal("The degree is larger than the total number of peers");
	}
	set_peers(num_peers);  // See MBWOverlay
	set_edges(degree);
}

// Set up a neighbourhood of randomly selected peers, with the degree of
// each peer being chosen from a poisson distribution (only not zero and
// not >= num_peers).
void GNPOverlay::set_edges(size_t degree_reqd)
{
	for (size_t i = 0; i < num_peers(); i++){
		size_t k = 0;
		size_t degree = 0;
		while(degree == 0 || degree > num_peers() - 1) {
			degree = poisson (degree_reqd, &gSeed);
		}
		while (k < degree) {
			size_t j = (size_t)floor(uniform (0, num_peers(), &gSeed));
			if (i != j){
				k++;
				if(find_link (i, j) == NULL){
					add_link (i, j);
					add_link (j, i);
				}
			}
		}
	}
}

