/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the GNROverlay, a subclass of MBWOverlay in which each
	peer is connected to a randomly-chosen fixed-size subset of the other
	peers.
 */

#ifndef _GNR_OVERLAY_H_
#define _GNR_OVERLAY_H_

#include "mbwoverlay.h"

/// A GNROverlay is an MBWOverlay in which each host is given
/// a randomly selected set of neighbours.
class GNROverlay : public MBWOverlay {
	size_t o_degree;
public:
	/// Constructor. Note that the degree of the overlay will be
	/// greater than the given degree parameter, typically by a factor approaching
	//  2, since the degree parameter just controls
	/// how many random neighbours are chosen. A chosen neighbour will
	/// be linked to the peer, unless it is already linked.
	GNROverlay(
		size_t num_peers,	///< Number of peers in overlay
		size_t degree		///< Number of random neighbours to select 
	);

	virtual void connect_peer(PPeer peer);

	/// Select a given percent of all peers (which must be AOPeers currently)
	/// as churners.
	void select_churners(double percent);
private:
	void set_edges(size_t degree);
};

#endif // _GNR_OVERLAY_H_
