/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Gloria Bogliolo, Ana Paula Couto,
		  Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Implementation of the LatestRandomPeer class.
 */

#include "latestrandompeer.h"


//================= LatestRandomPeer ======================
// This peer selects the most recent chunk (subject to there being existing
// neighbours who don't have it) and a random neighbour (of those who don't have it).
// It is a handcrafted version of the peer behaviour obtained from a 
// config_peer with configuration
// ChunkFirst BestChunk AllChunks Recentness BestPeer NeedsSelected UnitWtPeer

void LatestRandomPeer::schedule_new_chunk()
{
	if (chunk_trace) printf("T = %lf, peer %d schedule_new_chunk\n", gCurrentTime, id());
	if (num_links_out() > 0 && !p_upload_busy) {
		bool done = false;
		ChunkIterator it = window()->begin();	// Iterator over chunks, latest first
		while (it != window()->end() && !done) {
			PChunk chunk = *it;
	                if (chunk_trace) printf("\tchunk %d selected\n", chunk->id());
			PLink neighbour = choose_best_neighbour(NULL, &needs_chunk_and_not_busy, chunk, NULL);
			if (neighbour != NULL) {
	                        if (chunk_trace) printf("\tpeer %d selected\n", neighbour->to()->id());
				schedule_chunk(chunk, neighbour);
				done = true;
			}
			else {
				++it;
			}
		}
	}
}
