/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file misc.cpp
	Implementations of the functions declared in misc.h
 */

#include "misc.h"
#include "globals.h"
#include "propertymap.h"
#include <cstdio>
#include <cstdlib>
#include <algorithm>

// Issue an error message and exit
void fatal(std::string message) {
	std::printf("*** FATAL ERROR *** %s\n", message.c_str());
	exit(1);
}

// Try to open the given file for writing. The name is the name of a *property*,
// mapped to a file name, and with the extra filename prefix and suffix added.
// Return the file pointer if successful, else issue error message and exit.
FILE* check_open(std::string property_name) {
	if (!config.has_key(property_name)) {
		printf("Warning: file %s not created -- no such key\n", property_name.c_str());
		return NULL;
	}
	else {
		std::string filename = config.getString(property_name);
		std::string filename_prefix = config.getString("FilenamePrefix");
		std::string filename_suffix = config.getString("FilenameSuffix");
		std::string full_name = filename_prefix + filename + filename_suffix;
		FILE* fp = fopen(full_name.c_str(), "w");
		if (fp == NULL) {
			std::string message = "The file " + full_name + " cannot be opened. Perhaps the output directory doesn't exist?";
			fatal(message.c_str());
		}
		return fp;
	}
}

// Return a random vector of n size_t's in range min .. max inclusive.
std::vector<size_t> random_ints(size_t n, size_t min, size_t max) {
	std::vector<size_t> nums(max - min + 1);
	size_t index = 0;
	for (size_t i = min; i <= max; i++) {  // Make an array of all candidates
		nums[index++] = i;
	}
	std::random_shuffle(nums.begin(), nums.end());
	nums.resize(n);
	return nums;
}
