/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Marco Mellia, Gloria Bogliolo, Ana Paula Couto, Richard Lobb
  	      Telecomunication Networks Group
  	      Politecnico di Torino
  	      Torino, Italy
  	      mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the factory method "make_overlay" of the Overlay class.
 */

#include "overlay.h"
#include "asawareoverlay.h"
#include "fulloverlay.h"
#include "gnroverlay.h"
#include "gnpoverlay.h"
#include "gnkregoverlay.h"
#include "treeoverlay.h"
#include "prebuiltoverlay.h"
#include "misc.h"
#include "globals.h"

// The Overlay::make_overlay factory method
// ====================================

POverlay Overlay::make_overlay(size_t num_peers, size_t degree)
{
	POverlay po = NULL;
	switch (o_type) {
		case FULL:
			if (degree!=num_peers-1) fatal("with a full mesh, degree should always be num-peers-1"); //TODO: this should better be just a warning
			po = new FullOverlay(num_peers);
			break;
		case GNR:
			po = new GNROverlay(num_peers, degree);
			break;
		case GNP:
			po = new GNPOverlay(num_peers, degree);
			break;
		case GNKREG:
			po = new GNKRegOverlay(num_peers, degree);
			break;
		case TREE:
			po = new TreeOverlay(num_peers, degree, gMaxTreeDegree);
			break;
		case PREBUILT:
			po = new PrebuiltOverlay();
			break;
		case AS_AWARE:
            po = new ASAwareOverlay(num_peers, degree);
            break;
		default:
			fatal("Unknown overlay type");
	}
	return po;
}

