#calculate the index in the results file that separates input parameters from outputs
RESULTSINDEX=$(($(linehead_count)+1))

#parameters: plotstr, title, outfile_prefix
function gnuplot_cdf {
cat >$3.gnuplot <<INPUT_END
#set style data steps
set style data lines
set title "$2"
set xlabel $XLABEL
set ylabel $YLABEL

set terminal postscript eps color
set output "$3.eps"
plot \\
$1

set terminal png size 1024,768
set output "$3.png"
replot

INPUT_END

}

#parameter: prefix
function plot_cdf {
  eval PLOTTITLE2=\"$PLOTTITLE\"
  PLOTSTR=""

  for CSCHED in $CSCHEDs ; do
  PLOTSTR2=""
  for SCHED_ORDER in $SCHED_ORDERs ; do
  for PSCHED in $PSCHEDs ; do
    eval OUTPREFIX2=\"$OUTPREFIX\"
    TITLE=$OUTPREFIX2
    FILE="$OUTPREFIX2-$DELAY_CDF_FILE.out"
    if [ "$PLOTSTR" ] ; then PLOTSTR+=$',\\\n' ; fi
    PLOTSTR="$PLOTSTR \"$FILE\" $USING title \"$TITLE\""
    if [ "$PLOTSTR2" ] ; then PLOTSTR2+=$',\\\n' ; fi
    PLOTSTR2="$PLOTSTR2 \"$FILE\" $USING title \"$TITLE\""
  done;  
  done;  
  gnuplot_cdf "$PLOTSTR2" "$PLOTTITLE2" $1-$CSCHED
  done;  

  gnuplot_cdf "$PLOTSTR" "$PLOTTITLE2" $1 

  for PSCHED in $PSCHEDs ; do
  PLOTSTR2=""
  for SCHED_ORDER in $SCHED_ORDERs ; do
  for CSCHED in $CSCHEDs ; do
    eval OUTPREFIX2=\"$OUTPREFIX\"
    TITLE=$OUTPREFIX2
    FILE="$OUTPREFIX2-$DELAY_CDF_FILE.out"
    if [ "$PLOTSTR2" ] ; then PLOTSTR2+=$',\\\n' ; fi
    PLOTSTR2="$PLOTSTR2 \"$FILE\" $USING title \"$TITLE\""
  done;  
  done;  
  gnuplot_cdf "$PLOTSTR2" "$PLOTTITLE2" $1-$PSCHED
  done;  
}

#plot_one GNUPLOTFILE FILEBASE PLOTSTR
function plot_one {
cat >>$1 <<INPUT_END
set terminal postscript eps color
set output "$2.eps"
plot \\
$3

set terminal png size 1024,768
set output "$2.png"
replot

INPUT_END
}

#initialize the filter string describing a curve
function plotstring_init {
  PLOTSTR=""
}

#create gnuplot plot instructions for the given curve 
function plotstring_plot {
  plot_one "$GNUPLOTFILE" "$FILENAME" "$PLOTSTR"
}

#create a filter string for a given curve based on the current state of variables defined in LINEHEAD
function linehead_to_regexp {
  r="^"
  for p in ${LINEHEAD//\\t\$/ }; do
    if [[ $r != "^" ]]; then r=$r"\\t"; fi
    if [[ ${!p} ]]; then
      r=$r"${!p}"
    else
      r=$r"[^\t]+"
    fi
  done
  echo "$r"
}

#create title string for a given curve based on the current state of variables defined in LINEHEAD
function linehead_to_title {
  for p in ${LINEHEAD//\\t\$/ }; do
    if [[ ${!p} ]]; then
      case $p in
        SCHED_ORDER )
          if [[ $r ]]; then r=$r","; fi
          if [ "$SCHED_ORDER" = "C_P" ] ; then r=$r"$CSCHED/$PSCHED"; else r=$r"$PSCHED/$CSCHED"; fi ;;
        CSCHED ) ;;
        PSCHED ) ;;
        * )
          if [[ $r ]]; then r=$r","; fi
          r=$r"$p=${!p}" ;;
      esac
    fi
  done
  echo "$r"
}

#create gnuplot style plot string for a given curve based on the current state of variables defined in LINEHEAD,
#and concatenate it to previous plotstring.
function plotstring_concat {
  CASE=$(linehead_to_regexp)
  TITLE=$(linehead_to_title)
  FILE="< awk '/$CASE/' $TESTOUTALL"
  if [ "$PLOTSTR" ] ; then PLOTSTR+=$',\\\n' ; fi
  PLOTSTR="$PLOTSTR \"$FILE\" $USING title \"$TITLE\""
}

#prepare gnuplot script for a given X-Y with curves varying to the variables defined in hte last parameter
#parameter: plot_one ${PREFIX} ${XINDEX} ${XSHORTNAME} ${XLABEL} ${YINDEX} ${YSHORTNAME} ${YLABEL} ${CURVEVARS}
function plot_multiple {
  PREFIX=$1; XINDEX=$2; XSHORTNAME=$3; XLABEL=$4; YINDEX=$5; YSHORTNAME=$6; YLABEL=$7; CURVEVARS=$8

  USING="using $XINDEX:$YINDEX smooth unique"
  FILEBASE=${PREFIX}${XSHORTNAME}-${YSHORTNAME}

#write init part of gnuplot file
  GNUPLOTFILE=$FILEBASE.gnuplot
  eval PLOTTITLE2=\"$PLOTTITLE\"
cat >$GNUPLOTFILE <<INPUT_END
#set style data yerrorlines
#set style data lines
set style data linespoints
set datafile missing 'inf' #percentiles might look better with this. Without this, smooth unique doesn't work!
set title "$PLOTTITLE2"
set xlabel $XLABEL
set ylabel $YLABEL
set key horiz outside center bottom	#place the legend outside the graph, on the bottom with horizontal alignment
$GNUPLOT_XTRA
INPUT_END


  #prepare plot command for all curves
  FILENAMEs='\"$FILEBASE\"'
  cyclevariables FILENAME precall plotstring_init postcall plotstring_plot $CURVEVARS precall plotstring_concat

  #prepare "section" plots for each variable
  for i in $CURVEVARS ; do 
    FILENAMEs='\"$FILEBASE-$i-\$$i\"'
    cyclevariables $i FILENAME precall plotstring_init postcall plotstring_plot ${CURVEVARS/$i/} precall plotstring_concat
  done
}

#create gnuplot scripts for each results variable
#parameter: variables to use to differentiate curves
function plotfiles {
  CURVEVARS=$1  #variables to use to differentiate curves

if [[ $OUTPREFIX ]]; then
  YLABEL="\"CDF\""
  YSHORTNAME=cdf
  USING=""   #requires gnuplot stddev patch!
  plot_cdf ${PREFIX}${XSHORTNAME}-${YSHORTNAME}
else

  YLABEL="\"average delay [s]\""
  YSHORTNAME=avgdelay
  YINDEX=$(($RESULTSINDEX + 4))
  plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

  YLABEL="\"max delay [s]\""
  YSHORTNAME=maxdelay
  YINDEX=$(($RESULTSINDEX + 5))
  plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

  YLABEL="\"chunk loss ratio\""
  YSHORTNAME=chunkloss
  YINDEX=$(($RESULTSINDEX + 3))
  plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

  if [[ $PERCENTILE ]]; then
    YLABEL="\"percentile 90%\""
    YSHORTNAME=percentile90
    YINDEX=$(($RESULTSINDEX + 6))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

    YLABEL="\"percentile 95%\""
    YSHORTNAME=percentile95
    YINDEX=$(($RESULTSINDEX + 7))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

    YLABEL="\"percentile 99%\""
    YSHORTNAME=percentile99
    YINDEX=$(($RESULTSINDEX + 8))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"
  fi

  if [[ $PSNR_EVERY ]]; then
    YLABEL="\"PSNR\""
    YSHORTNAME=psnr
    YINDEX=$(($RESULTSINDEX + 11))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

    YLABEL="\"SSIM\""
    YSHORTNAME=ssim
    YINDEX=$(($RESULTSINDEX + 12))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

  fi

  if [[ $TIME_MEM ]]; then
    YLABEL="\"user time\""
    YSHORTNAME=utime
    YINDEX=$(($RESULTSINDEX + 17))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"

    YLABEL="\"maximum memory usage\""
    YSHORTNAME=maxmem
    YINDEX=$(($RESULTSINDEX + 20))
    plot_multiple "${PREFIX}" "${XINDEX}" "${XSHORTNAME}" "${XLABEL}" "${YINDEX}" "${YSHORTNAME}" "${YLABEL}" "${CURVEVARS}"
  fi
fi
}

#plot all pre-prepared gnuplot scrips
function plot {
  for GNUPLOT_FILE in `ls *.gnuplot` ; do
	gnuplot $GNUPLOT_FILE
  done;
}


