/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

 Authors: Maurizio Munafo', Marco Mellia, Richard Lobb, Gloria Bogliolo, 
/*********************************************************************/
/* Copyright (C) 2008  Politecnico di Torino

Authors:    Maurizio Munafo', Marco Mellia, Gloria Bogliolo, 
            Ana Paula Couto, Richard Lobb
			Telecomunication Networks Group
			Politecnico di Torino
			Torino, Italy
			mellia@mail.tlc.polito.it, anapaula@tlc.polito.it

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/
/*********************************************************************/

/*! @file
	Defines the implementation of the Router class. In the current implementation
	this is just a "glue" class between a host and the network, with no
	functionality of its own. See host.h for the specifications of the Router
	methods (such as they are).
 */

#include "router.h"
#include "peer.h"
#include "host.h"
#include "network.h"

void Router::set_id(RouterID id) {
	r_id = id;
}

RouterID Router::getId()
{
	return r_id;
}

void Router::set_network(PNetwork network) {
	r_pnetwork = network;
}

void Router::set_as(PAS as) {
	r_pas = as;
}

PNetwork Router::network() {
	return r_pnetwork;
}

HostID Router::add_host(PHost host) {
	HostID id = r_pnetwork->register_host(host);
	host->set_id(id);
	host->set_router(this);
	return id;
}

/// Notification to the network (via the entry point, the router)
/// that a host has initiated sending of a chunk.
/// Currently unused -- is a hook for future development.
void Router::sending_chunk(PChunk chunk, Path path)
{
	return;
}

/// Return the network latency, i.e. the additional delay (over and above
/// bandwidth-determined delay) on the given path, between the two routers only.
Latency Router::network_latency(Path path) {
	return r_pnetwork == NULL ? 0 : r_pnetwork->network_latency(path);
}
